@@ -46,7 +46,7 @@ from contextlib import contextmanager
 import msgpack
 import pylibmc
 
-VERSION = "0.6.5"
+VERSION = "0.6.6"
 LOCK_SH = 0
 LOCK_EX = 1
 RUN_PATH = "/run/starrydb/"
@@ -181,7 +181,8 @@ class LibMemcachedClient(pylibmc.ThreadMappedPool):
             return mc.delete(key)
 
 class StarryServer(object):
-    def __init__(self, address=None, depots=[], cache_size=0, log_file=None, rw_ratio=5):
+    def __init__(self, address=SERVER_ADDRESS, depots=[], cache_size=0, log_file=None, rw_ratio=5):
+        self.address = address
         self.cursor = 0
         self.depots = {}
         self.logger = StarryLog(log_file) if log_file is not None else None
@@ -190,12 +191,6 @@ class StarryServer(object):
 
         pathlib.Path(RUN_PATH).mkdir(exist_ok=True)
 
-        if address is None:
-            self.address = SERVER_ADDRESS
-        else:
-            self.address = address
-            pathlib.Path(SERVER_ADDRESS).write_bytes(msgpack.packb(address))
-
         if int(cache_size) > 0:
             try:
                 subprocess.check_output(f"fuser -k {CACHE_SOCKET}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
@@ -219,8 +214,11 @@ class StarryServer(object):
 
                 self.depots[depot_path] = StarryDepot(depot_path, self.logger, self.rw_ratio, self.cursor, self.memcached)
 
-                if depot.get("cache", True) is False:
-                    self.depots[depot_path].disableCache()
+                if depot.get("backup", False) is True:
+                    self.depots[depot_path].enableBackup()
+
+                if depot.get("cache", False) is True:
+                    self.depots[depot_path].enableCache()
 
                 self.cursor += 1
 
@@ -289,7 +287,8 @@ class StarryDepot(object):
         self.rw_ratio = rw_ratio
         self.index = index
         self.memcached = memcached
-        self.cache = memcached
+        self.cache = None
+        self.bak = False
         self.thread_cond = threading.Condition()
         self.thread_lock = threading.RLock()
         self.write_wait = 0
@@ -300,11 +299,6 @@ class StarryDepot(object):
         if not self.database.is_dir():
             raise StarryError(f"path {path} doesn't exist.")
 
-        if not self.database.joinpath(".backup").is_dir():
-            self.database.joinpath(".backup").mkdir(exist_ok=True)
-
-            self.backup()
-
     def executeCommand(self, conn, mode=LOCK_SH):
         if mode == LOCK_SH:
             while True:
@@ -361,6 +355,17 @@ class StarryDepot(object):
     def disableCache(self):
         self.cache = None
 
+    def enableBackup(self):
+        self.bak = True
+
+        if not self.database.joinpath(".backup").is_dir():
+            self.database.joinpath(".backup").mkdir(exist_ok=True)
+
+            self.backup()
+
+    def disableBackup(self):
+        self.bak = False
+
     def get(self, key):
         if type(key) is not list:
             key = [key]
@@ -510,14 +515,9 @@ class StarryDepot(object):
 
         with self._lock(mark):
             for key, value in values.items():
-                if self.database.joinpath(f"{key}.mpb").exists():
-                    (ret, msg) = self._update([key], value)
-                    if ret is False:
-                        return (ret, msg)
-                else:
-                    (ret, msg) = self._insert([key], value)
-                    if ret is False:
-                        return (ret, msg)
+                (ret, msg) = self._update([key], value)
+                if ret is False:
+                    return (ret, msg)
 
             for elem in self.database.glob("*"):
                 if elem.suffix == ".mpb":
@@ -548,11 +548,12 @@ class StarryDepot(object):
                     if ret is False:
                         self._recover(elem.stem)
 
-            for elem in self.database.joinpath(".backup").glob("*"):
-                if elem.suffix == ".mpb" and not self.database.joinpath(elem.name).exists():
-                    (ret, msg) = self._get([elem.stem])
-                    if ret is False:
-                        self._recover(elem.stem)
+            if self.database.joinpath(".backup").is_dir():
+                for elem in self.database.joinpath(".backup").glob("*"):
+                    if elem.suffix == ".mpb" and not self.database.joinpath(elem.name).exists():
+                        (ret, msg) = self._get([elem.stem])
+                        if ret is False:
+                            self._recover(elem.stem)
 
             return (True, "")
 
@@ -705,17 +706,7 @@ class StarryDepot(object):
         file = self.database.joinpath(f"{key}.mpb")
 
         if len(branch) == 0:
-            if not file.exists():
-                try:
-                    file.touch()
-                except Exception as e:
-                    try:
-                        file.unlink()
-                    except:
-                        pass
-
-                    return (False, f"Caught exception: {e.__doc__}({e})")
-            elif self._getCache(key) == part:
+            if self._getCache(key) == part:
                 return (True, "")
 
             value = part
@@ -723,10 +714,11 @@ class StarryDepot(object):
             if not file.exists():
                 return (False, f"No {key}.")
 
-            if not self.database.joinpath(".backup", f"{key}.mpb").exists():
-                (ret, msg) = self._backup(key)
-                if ret is False:
-                    return (ret, msg)
+            if self.bak is True:
+                if not self.database.joinpath(".backup", f"{key}.mpb").exists():
+                    (ret, msg) = self._backup(key)
+                    if ret is False:
+                        return (ret, msg)
 
             (ret, value) = _readFile(file)
             if ret is False:
@@ -749,6 +741,17 @@ class StarryDepot(object):
         except Exception as e:
             return (False, f"Caught exception: {e.__doc__}({e})")
 
+        if not file.exists():
+            try:
+                file.touch()
+            except Exception as e:
+                try:
+                    file.unlink()
+                except:
+                    pass
+
+                return (False, f"Caught exception: {e.__doc__}({e})")
+
         (ret, msg) = _writeFile(file, value)
         if ret is False:
             self._recover(key)
@@ -756,7 +759,10 @@ class StarryDepot(object):
 
         self._setCache(key, value)
 
-        return self._backup(key, check_flag=False)
+        if self.bak is True:
+            return self._backup(key, check_flag=False)
+
+        return (True, "")
 
     def _insert(self, route, part):
         branch = route[:]
@@ -769,10 +775,11 @@ class StarryDepot(object):
             if not file.exists():
                 return (False, f"No {key}.")
 
-            if not self.database.joinpath(".backup", f"{key}.mpb").exists():
-                (ret, msg) = self._backup(key)
-                if ret is False:
-                    return (ret, msg)
+            if self.bak is True:
+                if not self.database.joinpath(".backup", f"{key}.mpb").exists():
+                    (ret, msg) = self._backup(key)
+                    if ret is False:
+                        return (ret, msg)
 
             (ret, value) = _readFile(file)
             if ret is False:
@@ -813,7 +820,10 @@ class StarryDepot(object):
 
         self._setCache(key, value)
 
-        return self._backup(key, check_flag=False)
+        if self.bak is True:
+            return self._backup(key, check_flag=False)
+
+        return (True, "")
 
     def _delete(self, route):
         branch = route[:]
@@ -845,10 +855,11 @@ class StarryDepot(object):
                 self._recover(key)
                 return (False, f"No {key}.")
 
-        if not backup_file.exists():
-            (ret, msg) = self._backup(key)
-            if ret is False:
-                return (ret, msg)
+        if self.bak is True:
+            if not backup_file.exists():
+                (ret, msg) = self._backup(key)
+                if ret is False:
+                    return (ret, msg)
 
         if len(branch) == 0:
             self._deleteCache(key)
@@ -902,7 +913,10 @@ class StarryDepot(object):
 
         self._setCache(key, value)
 
-        return self._backup(key, check_flag=False)
+        if self.bak is True:
+            return self._backup(key, check_flag=False)
+
+        return (True, "")
 
     def _backup(self, key, check_flag=True, path=None):
         file = self.database.joinpath(f"{key}.mpb")
@@ -978,11 +992,12 @@ class StarryDepot(object):
         except Exception as e:
             return (False, f"Caught exception: {e.__doc__}({e})")
 
-        if file.exists():
-            if not backup_file.exists():
-                (ret, msg) = self._backup(key)
-                if ret is False:
-                    return (ret, msg)
+        if self.bak is True:
+            if file.exists():
+                if not backup_file.exists():
+                    (ret, msg) = self._backup(key)
+                    if ret is False:
+                        return (ret, msg)
 
         (ret, msg) = _copyFile(source_file, file)
         if ret is False:
@@ -1008,7 +1023,10 @@ class StarryDepot(object):
 
         self._setCache(key, value)
 
-        return self._backup(key, check_flag=False)
+        if self.bak is True:
+            return self._backup(key, check_flag=False)
+
+        return (True, "")
 
     @contextmanager
     def _lock(self, mark):
@@ -1072,9 +1090,6 @@ class StarryDepot(object):
 
 class StarryClient(object):
     def __init__(self, path, mode=LOCK_SH, address=SERVER_ADDRESS):
-        if address is None:
-            address = msgpack.unpackb(pathlib.Path(SERVER_ADDRESS).read_bytes(), use_list=False)
-
         self.client = multiprocessing.connection.Client(address, authkey=b'starry')
         self.client.send((os.path.abspath(path), mode))
 
